(reproducible) validation procedure to benchmark vbSPT
------------------------------------------------------

This folder contains all the ingredients to reproduce the benchmark of `vbSPT` against data simulated by `simSPT`. `vbSPT` is described [elsewhere](http://vbspt.sourceforge.net/). Basically, `vbSPT` infers the diffusion coefficients and respective fractions of SPT data and performs the classification of individual trajectories (including segments).

In this folder, we do not use the classification of trajectories, but only the estimates of diffusion coefficients and fractions at the pooled level. Furthermore, we restrict the analysis to the inference of up to 2 states. That is that the routine can return either one or two states.

The simulated data are described elsewhere (in the `simSPT_simulations` folder).

# Overview
To analyze one dataset using `vbSPT`, one has to generate a matlab file containing the fitting parameters and associated input/output paths (*command file*). Thus, running `vbSPT` on a series of 3480 simulations requires to generate one *command file* per simulation, and then run them in parallel. Furthermore, the files have to be converted in the `vbSPT` format. The instructions to perform this conversion lies in the *convert file*.

The scripts in this folder perform the following tasks. They have to be run in order:

1. Convert the CSV files to the Matlab format used in input of all the scripts (`ConvertSimsToMAT_par.m`)
2. Generate the *command file* and the *convert file* (`cluster_missing.py`).
3. Finally run all the computations in parallel (`main.sh`)

# Usage
## Prerequities
1. A working installation of bash, Python, and Matlab. It is recommended to test that `vbSPT` functions properly before running the set of simulations.

2. To download the simulations and store them in the csv folder. They should be in the CSV folder.

## Convert the CSV to .mat files

1. Edit the `ConvertSimsToMAT_par.m` file
   1. Make sure that `csv_dir` points to the folder containing the csv files from the simulations.
   2. Make sure that the `out_dir` directory exists and is empty
   3. Adjust the number of cores of your machines in the `parpool('local', 31);` instruction. Here, we configure Matlab to use 31 cores. It is recommended to use the number of cores of your machine.
2. Run the conversion: in Matlab, run the script `ConvertSimsToMAT_par`. This should take a while.

When done, you should have the same number of files in the `csv` and in the `mat` folder.

## Generate the list of conversions to perform
The script below will extract from the name of the mat file the framerate, and will generate one *command* file and one *convert* file per dataset in the `mat` folder.

To do so, the script will use the template files in the `template` folder.

1. Edit the `templates/convert_template.m` file and make sure that all the paths are properly filled in:
   1. `path2` is the path that contains the `tmp/` subfolder where intermediate files will be saved
   2. `currPath` is the path where the previously converted files are (mat files)
   3. `dir0` the path to the `vbSPT` program
   
2. Edit the `templates/command_template.m` file if needed (if you want to adjust the number of states, etc. This file follows the standard vbSPT syntax.

3. Edit `cluster_missing.py` and make sure that the folder point to existing directory. `in_folder` should point to the location of the converted matlab files. `out_folder` should be consistent with `path2`. `command_folder` and `convert_folder` should exist, `convert_template_path` and `command_template_path` should point to the previously mentioned `templates/convert_template.m` and `templates/command_template.m`, respectively. `tp` should be the absolute path of the the `convert_files` directory.

4. Run the script to generate the *command* files and *convert* files. Note that we store the output in the `tocompute.txt`, and this file will be given to Matlab. To do so, type (in a shell): `python cluster_missing.py > tocompute.txt`

4. Check that files were created in the `convert_files` and `command_files` directories, and that the `tocompute.txt` file contains the appropriate instructions. There should be one line per simulation. 

## Perform the computation
Once these preliminary steps have been performed, performing the fitting should be pretty straightforward:

1. Edit `main.sh`: the number after the `-j` flag is the number of cores over which to perform the parallelization.
2. Run it (in bash): `./main.sh > logfile.log`
3. This should create a series of .mat files in the `results/` folder.

## Extract the coefficients

Run:

```{shell}
echo ",D1,D1_ref,D2,D2_ref,Nstates,P1,P1_ref,dt,file" > AllResults.csv
ls results/ | parallel -j 8 eval python to_csv_missing2.py {} > AllResults.csv
```

`AllResults.csv` should now contain the extracted coefficients.
